#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
class Constants:
    NAME = "name"
    MESSAGE_ID = "message_id"
    CALLBACK_TOPIC = "callback_topic"
    KAFKA_ERROR = "kafka_error"
    ID_TOKEN = "id_token"

    PEER = "peer"

    ACTOR_GUID = "actor_guid"
    MODE = "mode"

    FAILED = "failed"
    MESSAGE = "message"

    RTYPE = "rtype"
    PARENT_ID = "parent_id"
    STATE = "state"
    SEQUENCE = "sequence"
    RESERVATION_ID = "reservation_id"
    SLICE_ID = "slice_id"
    ACTOR_ID = "actor_id"
    PROPERTIES = "properties"
    SLIVER = "sliver"
    MAINT_INFO = "maint_info"
    POAS = "poas"
    INFO = "info"
    VCPU_CPU_MAP = "vcpu_cpu_map"
    NODE_SET = "node_set"
    KEYS = "keys"
    BDF = "bdf"
    BROKER = "broker"
    TICKET = "ticket"
    RENEWABLE = "renewable"
    RENEW_TIME = "renew_time"
    UNIT_SET = "unit_set"
    UNITS = "units"
    PROXIES = "proxies"
    DELEGATIONS = "delegations"
    ACTORS = "actors"
    MODEL = "model"
    AUTHORITY = "authority"

    RESERVATION = "reservation"
    RESERVATIONS = "reservations"
    RESERVATION_STATES = "reservation_states"
    AUTH = "auth"

    TYPE = "type"
    OWNER = "owner"
    DESCRIPTION = "description"
    POLICY_CLASS = "policy_class"
    POLICY_MODULE = "policy_module"
    ACTOR_CLASS = "actor_class"
    ACTOR_MODULE = "actor_module"
    ONLINE = "online"
    MANAGEMENT_CLASS = "management_class"
    MANAGEMENT_MODULE = "management_module"
    ID = "id"
    ATTEMPT = "attempt"
    POLICY_GUID = "policy_guid"

    RESERVATION_LIST = "reservation_list"
    RESERVATION_OBJ = "reservation_obj"
    SLICE_OBJ = "slice_obj"
    SLICE = "slice"
    SLICES = "slices"
    TERM = "term"
    RESOURCE_SET = "resource_set"
    RESOURCE_TICKET = "resource_ticket"

    UPDATE_DATA = "update_data"
    DELEGATION = "delegation"
    STATUS = "status"
    SITES = "sites"

    REDEEM_PREDECESSORS = "redeem_processors"

    ERROR_INVALID_ARGUMENTS = "Invalid arguments"
    ERROR_INVALID_MESSAGE = "Invalid message"



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from __future__ import annotations

import pickle
from typing import List

from fabric_mb.message_bus.messages.constants import Constants
from fabric_mb.message_bus.messages.reservation_predecessor_avro import ReservationPredecessorAvro
from fabric_mb.message_bus.messages.ticket_reservation_avro import TicketReservationAvro


class LeaseReservationAvro(TicketReservationAvro):
    """
    Implements Avro representation of a Lease Reservation
    """
    def __init__(self):
        super(LeaseReservationAvro, self).__init__()
        self.authority = None
        self.join_state = None
        self.leased_units = None
        self.redeem_processors = []
        self.name = self.__class__.__name__

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.REDEEM_PREDECESSORS:
                    for pred in v:
                        predecessor = ReservationPredecessorAvro()
                        predecessor.from_dict(value=pred)
                        self.redeem_processors.append(predecessor)
                elif k == Constants.SLIVER and v is not None:
                    self.__dict__[k] = pickle.loads(v)
                else:
                    self.__dict__[k] = v

    def print(self):
        """
        Print reservation on console
        Used by management cli
        """
        print("")
        print("Reservation ID: {} Slice ID: {}".format(self.reservation_id, self.slice_id))
        if self.rtype is not None or self.notices is not None:
            print("Resource Type: {} Notices: {}".format(self.rtype, self.notices))

        if self.start is not None or self.end is not None or self.requested_end is not None:
            print("Start: {} End: {} Requested End: {}".format(self.start, self.end, self.requested_end))

        if self.units is not None or self.state is not None or self.pending_state is not None:
            print("Units: {} State: {} Pending State: {}".format(self.units, self.state, self.pending_state))

        print("Broker: {}".format(self.broker))

        if self.ticket is not None:
            print("ticket properties: {}".format(self.ticket))

        if self.renewable is not None:
            print("Renewable: {}".format(self.renewable))

        if self.renew_time is not None:
            print("Renew Time: {}".format(self.renew_time))

        print("Authority: {}".format(self.authority))

        if self.join_state is not None:
            print("Join State: {}".format(self.join_state))

        if self.leased_units is not None:
            print("Leased Units: {}".format(self.leased_units))

        if self.redeem_processors is not None:
            index = 0
            for rp in self.redeem_processors:
                print("redeem Predecessor# {}: {}".format(index, rp))
                index += 1

        if self.local is not None:
            print("Local Properties: {}".format(self.local))
        if self.config is not None:
            print("Config Properties: {}".format(self.config))
        if self.request is not None:
            print("Request Properties: {}".format(self.request))
        if self.resource is not None:
            print("Resource Properties: {}".format(self.resource))
        print("")

    def get_authority(self) -> str:
        """
        Return authority
        @return authority
        """
        return self.authority

    def set_authority(self, value: str):
        """
        Set authority
        @param value value
        """
        self.authority = value

    def get_join_state(self) -> int:
        """
        Return Join State
        @return join state
        """
        return self.join_state

    def set_join_state(self, value: int):
        """
        Set join state
        @param value value
        """
        self.join_state = value

    def get_leased_units(self) -> int:
        """
        Return number of leased units
        @return leased units
        """
        return self.leased_units

    def set_leased_units(self, value: int):
        """
        Set leased units
        @param value value
        """
        self.leased_units = value

    def get_redeem_predecessors(self) -> List[ReservationPredecessorAvro]:
        """
        Return redeem processors
        @return redeem processors
        """
        return self.redeem_processors



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro


class ResultAvro(AbcObjectAvro):
    """
    Implements Avro representation of a Result Message Status
    """
    def __init__(self):
        self.code = 0
        self.message = ""
        self.details = ""

    def get_code(self) -> int:
        """
        Return Status code
        """
        return self.code

    def set_code(self, code: int):
        """
        Set status code
        @param code code
        """
        self.code = code

    def get_message(self) -> str:
        """
        Return status message
        @return status message
        """
        return self.message

    def set_message(self, msg: str):
        """
        Set status message
        @param msg msg
        """
        self.message = msg

    def get_details(self) -> str:
        """
        Return details
        @return details
        """
        return self.details

    def set_details(self, value: str):
        """
        Set details
        @param value value
        """
        self.details = value

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.code is None:
            ret_val = False
        return ret_val



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from typing import List
from datetime import datetime, timezone

from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro


class RequestByIdRecord(AbcMessageAvro):
    """
    Implements Avro representation of a Get Request Message
    """

    def __init__(self):
        super(RequestByIdRecord, self).__init__()
        self.guid = None
        self.slice_id = None
        self.reservation_id = None
        self.delegation_id = None
        self.type = None
        self.unit_id = None
        self.broker_id = None
        self.reservation_state = None
        self.delegation_state = None
        self.auth = None
        self.id_token = None
        self.slice_name = None
        self.level = None
        self.email = None
        self.graph_format = None
        self.site = None
        self.states = None
        self.start = None
        self.end = None
        self.ip_subnet = None
        self.host = None

    def get_slice_id(self) -> str:
        """
        Returns the slice_id
        """
        return self.slice_id

    def get_reservation_id(self) -> str:
        """
        Returns the reservation if
        """
        return self.reservation_id

    def get_delegation_id(self) -> str:
        """
        Returns the delegation id
        """
        return self.delegation_id

    def get_type(self) -> str:
        """
        Returns the type
        """
        return self.type

    def get_unit_id(self) -> str:
        """
        Returns the unit_id
        """
        return self.unit_id

    def get_broker_id(self) -> str:
        """
        Returns the broker_id
        """
        return self.broker_id

    def get_reservation_state(self) -> int:
        """
        Returns the reservation_state
        """
        return self.reservation_state

    def get_states(self) -> List[int]:
        return self.states

    def get_delegation_state(self) -> int:
        """
        Returns the delegation_state
        """
        return self.delegation_state

    def get_id_token(self) -> str:
        """
        Returns the id token
        """
        return self.id_token

    def get_slice_name(self) -> str:
        return self.slice_name

    def get_level(self) -> int:
        return self.level

    def set_level(self, value):
        self.level = value

    def set_email(self, email: str):
        self.email = email

    def get_email(self) -> str:
        return self.email

    def get_graph_format(self) -> int:
        return self.graph_format

    def set_graph_format(self, graph_format):
        self.graph_format = graph_format

    def set_site(self, site: str):
        self.site = site

    def get_site(self) -> str:
        return self.site

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = super().validate()

        if self.guid is None or self.auth is None or self.callback_topic is None:
            ret_val = False

        return ret_val

    def set_end(self, end: datetime):
        if end is not None:
            self.end = int(end.timestamp())

    def set_start(self, start: datetime):
        if start is not None:
            self.start = int(start.timestamp())

    def get_end(self) -> datetime:
        if self.end is not None:
            return datetime.fromtimestamp(self.end, tz=timezone.utc)

    def get_start(self) -> datetime:
        if self.start is not None:
            return datetime.fromtimestamp(self.start, tz=timezone.utc)

    def set_ip_subnet(self, ip_subnet: str):
        self.ip_subnet = ip_subnet

    def get_ip_subnet(self) -> str:
        return self.ip_subnet

    def set_host(self, host: str):
        self.host = host

    def get_host(self) -> str:
        return self.host




#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro
from fabric_mb.message_bus.messages.auth_avro import AuthAvro
from fabric_mb.message_bus.messages.constants import Constants


class ActorAvro(AbcObjectAvro):
    """
    Implements Avro representation of an Actor
    """
    def __init__(self):
        self.name = None
        self.type = None
        self.owner = None
        self.description = None
        self.policy_class = None
        self.policy_module = None
        self.actor_class = None
        self.actor_module = None
        self.online = None
        self.management_class = None
        self.management_module = None
        self.id = None
        self.policy_guid = None

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.OWNER:
                    self.__dict__[k] = AuthAvro()
                    self.__dict__[k].from_dict(value=v)
                else:
                    self.__dict__[k] = v

    def get_name(self) -> str:
        """
        Returns the name
        """
        return self.name

    def set_name(self, value: str):
        """
        Set name
        @param value value
        """
        self.name = value

    def set_type(self, value: int):
        """
        Set type
        @param value value
        """
        self.type = value

    def get_type(self) -> int:
        """
        Get Type
        @return type
        """
        return self.type

    def set_owner(self, value: AuthAvro):
        """
        Set owner
        @param value value
        """
        self.owner = value

    def get_owner(self) -> AuthAvro:
        """
        Return owner
        @return owner
        """
        return self.owner

    def set_description(self, value: str):
        """
        Set description
        @param value value
        """
        self.description = value

    def get_description(self) -> str:
        """
        Get description
        @return description
        """
        return self.description

    def get_policy_class(self) -> str:
        """
        Get Policy Class name
        @return policy class name
        """
        return self.policy_class

    def set_policy_class(self, value: str):
        """
        Set policy class name
        @param value value
        """
        self.policy_class = value

    def get_policy_module(self) -> str:
        """
        Get Policy module name
        @return policy module name
        """
        return self.policy_module

    def set_policy_module(self, value: str):
        """
        Set policy module name
        @param value value
        """
        self.policy_module = value

    def get_actor_class(self) -> str:
        """
        Get actor class name
        @return actor class name
        """
        return self.actor_class

    def set_actor_class(self, value: str):
        """
        Set actor class name
        @param value value
        """
        self.actor_class = value

    def get_actor_module(self) -> str:
        """
        Get actor module name
        @return actor module name
        """
        return self.actor_module

    def set_actor_module(self, value: str):
        """
        Set actor module name
        @param value value
        """
        self.actor_module = value

    def set_online(self, value: bool):
        """
        Set online
        @param value value
        """
        self.online = value

    def get_online(self) -> bool:
        """
        Get online
        @return online status
        """
        return self.online

    def get_management_class(self) -> str:
        """
        Get Management class
        """
        return self.management_class

    def set_management_class(self, value: str):
        """
        Set management class name
        @param value value
        """
        self.management_class = value

    def get_management_module(self) -> str:
        """
        Get management module
        """
        return self.management_module

    def set_management_module(self, value: str):
        """
        Set management module name
        @param value value
        """
        self.management_module = value

    def set_id(self, value: str):
        """
        Set id
        @param value value
        """
        self.id = value

    def get_id(self) -> str:
        """
        Return id
        @return id
        """
        return self.id

    def set_policy_guid(self, value: str):
        """
        Set policy guid
        @param value value
        """
        self.policy_guid = value

    def get_policy_guid(self) -> str:
        """
        Get policy guid
        """
        return self.policy_guid

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.name is None or self.owner is None or self.description is None or self.policy_module is None or \
            self.policy_class is None or self.policy_guid is None or self.actor_class is None or \
                self.actor_module is None or self.id is None:
            ret_val = False
        return ret_val



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import pickle
from abc import ABC, abstractmethod
from fabric_mb.message_bus.message_bus_exception import MessageBusException
from fabric_mb.message_bus.messages.constants import Constants


class AbcObjectAvro(ABC):
    def to_dict(self) -> dict:
        """
        The Avro Python library does not support code generation.
        For this reason we must provide a dict representation of our class for serialization.
        :return dict representing the class
        """
        if not self.validate():
            raise MessageBusException(Constants.ERROR_INVALID_ARGUMENTS)

        result = self.__dict__.copy()

        for k in self.__dict__:
            if result[k] is None:
                result.pop(k)
            elif k in [Constants.SLIVER, Constants.MAINT_INFO, Constants.INFO]:
                v = result[k]
                if v is not None:
                    result[k] = pickle.dumps(v)
            elif isinstance(result[k], AbcObjectAvro):
                result[k] = result[k].to_dict()
            elif isinstance(result[k], list):
                temp = []
                for elem in result[k]:
                    temp.append(elem.to_dict())
                result[k] = temp

        return result

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__:
                if (k in [Constants.SLIVER, Constants.MAINT_INFO, Constants.INFO]) and v is not None:
                    self.__dict__[k] = pickle.loads(v)
                elif k == Constants.AUTH:
                    from fabric_mb.message_bus.messages.auth_avro import AuthAvro
                    self.__dict__[k] = AuthAvro()
                    self.__dict__[k].from_dict(value=v)
                else:
                    self.__dict__[k] = v

    def __str__(self):
        d = self.__dict__.copy()
        for k in self.__dict__:
            if d[k] is None:
                d.pop(k)
        if len(d) == 0:
            return ''
        ret = "{ "
        for i, v in d.items():
            ret = f"{ret} {i}: {v},"
        return ret[:-1] + "}"

    @abstractmethod
    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """

    def __eq__(self, other):
        assert isinstance(other, AbcObjectAvro)
        for f, v in self.__dict__.items():
            if v != other.__dict__[f]:
                return False
        return True



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from datetime import datetime, timezone

from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro
from fabric_mb.message_bus.messages.auth_avro import AuthAvro
from fabric_mb.message_bus.messages.constants import Constants


class SliceAvro(AbcObjectAvro):
    """
    Implements Avro representation of a Slice
    """
    def __init__(self):
        self.slice_name = None
        self.guid = None
        self.owner = None
        self.description = None
        self.config_properties = None
        self.resource_type = None
        self.client_slice = False
        self.broker_client_slice = False
        self.graph_id = None
        self.state = None
        self.inventory = False
        self.lease_start = None
        self.lease_end = None
        self.project_id = None
        self.project_name = None

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.OWNER:
                    self.__dict__[k] = AuthAvro()
                    self.__dict__[k].from_dict(value=v)
                else:
                    self.__dict__[k] = v

    def print(self, all: bool = False):
        """
        Print object on console
        Used by management CLI
        """
        print("")
        print("Slice Name: {} Slice Guid: {}".format(self.slice_name, self.guid))
        if self.graph_id is not None:
            print("Graph ID: {}".format(self.graph_id))
        if all:
            if self.project_id is not None:
                print("Project ID: {}".format(self.project_id))
            if self.project_name is not None:
                print("Project Name: {}".format(self.project_name))
            if self.owner is not None:
                print(f"Slice owner: {self.owner}")
            if self.description is not None:
                print(f"Description: {self.description}")
            if self.resource_type is not None:
                print(f"resource_type: {self.resource_type}")
            if self.client_slice is not None:
                print(f"client_slice: {self.client_slice}")
            if self.broker_client_slice is not None:
                print(f"broker_client_slice: {self.broker_client_slice}")
            if self.inventory is not None:
                print(f"inventory: {self.inventory}")
            if self.config_properties is not None:
                print(f"config_properties: {self.config_properties}")
        print("")

    def set_slice_name(self, value: str):
        """
        Set slice name
        @param value value
        """
        self.slice_name = value

    def get_slice_name(self) -> str:
        """
        Get Slice name
        @return slice name
        """
        return self.slice_name

    def set_owner(self, value: AuthAvro):
        """
        Set owner
        @param value value
        """
        self.owner = value

    def get_owner(self) -> AuthAvro:
        """
        Get Slice owner
        @return slice owner
        """
        return self.owner

    def set_description(self, value: str):
        """
        Set description
        @param value value
        """
        self.description = value

    def get_description(self) -> str:
        """
        Get Slice description
        @return slice description
        """
        return self.description

    def get_config_properties(self) -> dict:
        """
        Get config properties
        @return config properties
        """
        return self.config_properties

    def set_config_properties(self, value: dict):
        """
        Set config properties
        @param value value
        """
        self.config_properties = value

    def get_resource_type(self) -> str:
        """
        Get resource type
        @return resource type
        """
        return self.resource_type

    def set_resource_type(self, value: str):
        """
        Set resource type
        @param value value
        """
        self.resource_type = value

    def is_client_slice(self):
        """
        Return True if slice is client slice else False
        """
        return self.client_slice

    def set_client_slice(self, value: bool):
        """
        Set slice is client slice
        @param value value
        """
        self.client_slice = value

    def is_broker_client_slice(self):
        """
        Return True if slice is broker client slice else False
        """
        return self.broker_client_slice

    def set_broker_client_slice(self, value: bool):
        """
        Set slice as broker client slice
        @param value value
        """
        self.broker_client_slice = value

    def get_slice_id(self) -> str:
        """
        Return slice id
        @return slice id
        """
        return self.guid

    def set_slice_id(self, slice_id: str):
        """
        Set slice id
        @param slice_id slice_id
        """
        self.guid = slice_id

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.slice_name is None or self.owner is None or self.description is None or \
                self.broker_client_slice is None or self.guid is None:
            ret_val = False
            print(f"name {self.slice_name} owner {self.owner} description: {self.description} broker_client_slice: "
                  f"{self.broker_client_slice} guid: {self.guid}")
        return ret_val

    def get_graph_id(self) -> str:
        """
        Return graph id
        @return graph id
        """
        return self.graph_id

    def get_state(self) -> int:
        """
        Return state
        @return state
        """
        return self.state

    def get_inventory(self) -> bool:
        return self.inventory

    def set_lease_end(self, lease_end: datetime):
        if lease_end is not None:
            self.lease_end = int(lease_end.timestamp())

    def set_lease_start(self, lease_start: datetime):
        if lease_start is not None:
            self.lease_start = int(lease_start.timestamp())

    def get_lease_end(self) -> datetime or None:
        if self.lease_end is not None:
            return datetime.fromtimestamp(self.lease_end, tz=timezone.utc)
        return self.lease_end

    def get_lease_start(self) -> datetime or None:
        if self.lease_start is not None:
            return datetime.fromtimestamp(self.lease_start, tz=timezone.utc)
        return self.lease_start

    def get_project_id(self) -> str:
        """
        Return project id
        @return project id
        """
        return self.project_id

    def set_project_id(self, project_id: str):
        """
        Set project id
        @param project_id project id
        """
        self.project_id = project_id

    def get_project_name(self) -> str:
        """
        Return project name
        @return project name
        """
        return self.project_name

    def set_project_name(self, project_name: str):
        """
        Set project name
        @param project_name project name
        """
        self.project_name = project_name


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import pickle

from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro
from fabric_mb.message_bus.messages.auth_avro import AuthAvro


class PoaInfoAvro(AbcObjectAvro):
    """
    Implements Avro representation of a query Message
    """
    def __init__(self, *, operation: str = None, info=None, rid: str = None, auth: AuthAvro = None,
                 poa_id: str = None, project_id: str = None, slice_id: str = None, state: str = None,
                 error: str = None):
        self.operation = operation
        self.info = info
        self.rid = rid
        self.auth = auth
        self.poa_id = poa_id
        self.project_id = project_id
        self.slice_id = slice_id
        self.state = state
        self.error = error

    def get_info(self) -> dict:
        """
        Return Cpu Info
        """
        if self.info is not None:
            if not isinstance(self.info, dict):
                self.info = pickle.loads(self.info)
        return self.info

    def get_operation(self) -> str:
        return self.operation

    def get_rid(self) -> str:
        return self.rid

    def get_project_id(self) -> str:
        return self.project_id

    def get_auth(self) -> AuthAvro:
        return self.auth

    def get_poa_id(self) -> str:
        return self.poa_id

    def get_slice_id(self) -> str:
        return self.slice_id

    def get_state(self) -> str:
        return self.state

    def get_error(self) -> str:
        return self.error

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.operation is None:
            ret_val = False
        return ret_val



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import logging
import traceback
from abc import ABC


class ABCMbApi(ABC):
    def __init__(self, *, logger: logging.Logger = None):
        self.logger = logger
        if logger is None:
            self.logger = logging.getLogger(__name__)
            logging.basicConfig(level=logging.DEBUG,
                                format="%(asctime)s [%(filename)s:%(lineno)d] [%(levelname)s] %(message)s",
                                handlers=[logging.StreamHandler()])

    def load_schema(self, schema_file: str):
        try:
            from confluent_kafka import avro
            file = open(schema_file, "r")
            schema_bytes = file.read()
            file.close()
            return avro.loads(schema_bytes)
        except Exception as e:
            self.logger.error(f"Exception occurred while loading the schema: {schema_file}: {e}")
            self.logger.error(traceback.format_exc())


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import pickle
from typing import List, Dict
from uuid import uuid4

from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro
from fabric_mb.message_bus.messages.auth_avro import AuthAvro


class PoaAvro(AbcMessageAvro):
    """
    Implements Avro representation of a query Message
    """
    def __init__(self, *, operation: str = None, vcpu_cpu_map=None, node_set=None, bdf = None,
                 callback_topic: str = None, rid: str = None, project_id: str = None, id_token: str = None,
                 message_id: str = None, auth: AuthAvro = None, poa_id: str = None, sequence: int = None,
                 slice_id: str = None, keys=None):
        super(PoaAvro, self).__init__(callback_topic=callback_topic, id_token=id_token,
                                      name=AbcMessageAvro.poa, message_id=message_id)
        self.operation = operation
        self.auth = auth
        self.rid = rid
        self.vcpu_cpu_map = vcpu_cpu_map
        self.node_set = node_set
        self.bdf = bdf
        self.keys = keys
        self.poa_id = poa_id
        self.project_id = project_id
        self.sequence = sequence
        self.slice_id = slice_id
        if self.poa_id is None:
            self.poa_id = uuid4().__str__()

    def get_bdf(self) -> List[str]:
        if self.bdf is not None:
            if not isinstance(self.bdf, list):
                self.bdf = pickle.loads(self.bdf)
        return self.bdf

    def get_keys(self) -> List[Dict[str, str]]:
        if self.keys is not None:
            if not isinstance(self.keys, List):
                self.keys = pickle.loads(self.keys)
        return self.keys

    def get_node_set(self) -> List[str]:
        if self.node_set is not None:
            if not isinstance(self.node_set, List):
                self.node_set = pickle.loads(self.node_set)
        return self.node_set

    def get_vcpu_cpu_map(self) -> List[Dict[str, str]]:
        if self.vcpu_cpu_map is not None:
            if not isinstance(self.vcpu_cpu_map, List):
                self.vcpu_cpu_map = pickle.loads(self.vcpu_cpu_map)
        return self.vcpu_cpu_map

    def get_operation(self) -> str:
        return self.operation

    def get_poa_id(self) -> str:
        return self.poa_id

    def get_rid(self) -> str:
        return self.rid

    def get_project_id(self) -> str:
        return self.project_id

    def get_sequence(self) -> int:
        return self.sequence

    def get_slice_id(self) -> str:
        return self.slice_id

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = super().validate()
        if self.callback_topic is None or self.operation is None:
            ret_val = False
        return ret_val



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric_mb.message_bus.message_bus_exception import MessageBusException
from fabric_mb.message_bus.messages.constants import Constants
from fabric_mb.message_bus.messages.reservation_mng import ReservationMng


class TicketReservationAvro(ReservationMng):
    """
    Implements Avro representation of a ticket Reservation from Management Interface
    """
    def __init__(self):
        super(TicketReservationAvro, self).__init__()
        self.broker = None
        self.ticket = None
        self.renewable = None
        self.renew_time = None
        self.name = self.__class__.__name__

    def print(self):
        print("")
        print("Reservation ID: {} Slice ID: {}".format(self.reservation_id, self.slice_id))
        if self.rtype is not None or self.notices is not None:
            print("Resource Type: {} Notices: {}".format(self.rtype, self.notices))

        if self.start is not None or self.end is not None or self.requested_end is not None:
            print("Start: {} End: {} Requested End: {}".format(self.start, self.end, self.requested_end))

        if self.units is not None or self.state is not None or self.pending_state is not None:
            print("Units: {} State: {} Pending State: {}".format(self.units, self.state, self.pending_state))

        print("Broker: {}".format(self.broker))

        if self.ticket is not None:
            print("ticket properties: {}".format(self.ticket))

        if self.renewable is not None:
            print("Renewable: {}".format(self.renewable))

        if self.renew_time is not None:
            print("Renew Time: {}".format(self.renew_time))

        if self.local is not None:
            print("Local Properties: {}".format(self.local))
        if self.config is not None:
            print("Config Properties: {}".format(self.config))
        if self.request is not None:
            print("Request Properties: {}".format(self.request))
        if self.resource is not None:
            print("Resource Properties: {}".format(self.resource))
        print("")

    def get_broker(self) -> str:
        """
        Returns broker
        @return broker
        """
        return self.broker

    def set_broker(self, value: str):
        """
        Set broker
        @param value value
        """
        self.broker = value

    def get_ticket_properties(self) -> dict:
        """
        Returns ticket properties
        @return dict
        """
        return self.ticket

    def set_ticket_properties(self, value: dict):
        """
        Set ticket properties
        @param value value
        """
        self.ticket = value

    def is_renewable(self) -> bool:
        """
        Returns true if renewable else False
        @return true if renewable else False
        """
        return self.renewable

    def set_renewable(self, value: bool):
        """
        Set renewable
        @param value value
        """
        self.renewable = value

    def get_renew_time(self) -> int:
        """
        Return renew time
        @return renew time
        """
        return self.renew_time

    def set_renew_time(self, value: int):
        """
        Set renew time
        @param value value
        """
        self.renew_time = value



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import pickle
from abc import ABC
from uuid import uuid4

from fabric_mb.message_bus.message_bus_exception import MessageBusException
from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro
from fabric_mb.message_bus.messages.auth_avro import AuthAvro
from fabric_mb.message_bus.messages.constants import Constants


class AbcMessageAvro(ABC):
    """
        Implements the base class for storing the deserialized Avro record. Each message is uniquely identified by
        a globally unique identifier. It must be inherited to include Actor specific fields and to_dict implementation.
        New Avro schema must be defined as per the inherited class and should be used for the producer/consumer
    """
    """
    NAMING Convention: Respective Class Name for each message should look like: <Message Name>Avro
    """
    claim_delegation = "ClaimDelegation"
    reclaim_delegation = "ReclaimDelegation"
    close = "Close"
    extend_lease = "ExtendLease"
    extend_ticket = "ExtendTicket"
    failed_rpc = "FailedRpc"
    modify_lease = "ModifyLease"
    poa = "Poa"
    query = "Query"
    query_result = "QueryResult"
    redeem = "Redeem"
    relinquish = "Relinquish"
    update_lease = "UpdateLease"
    update_ticket = "UpdateTicket"
    ticket = "Ticket"
    update_delegation = "UpdateDelegation"

    # Management APIs
    claim_resources = "ClaimResources"
    reclaim_resources = "ReclaimResources"
    remove_slice = "RemoveSlice"
    add_slice = "AddSlice"
    update_slice = "UpdateSlice"
    remove_delegation = "RemoveDelegation"
    remove_reservation = "RemoveReservation"
    close_delegations = "CloseDelegations"
    close_reservations = "CloseReservations"
    update_reservation = "UpdateReservation"
    add_reservation = "AddReservation"
    add_reservations = "AddReservations"
    demand_reservation = "DemandReservation"
    extend_reservation = "ExtendReservation"
    add_peer = "AddPeer"
    maintenance_request = "MaintenanceRequest"

    get_reservations_state_request = "GetReservationsStateRequest"
    get_slices_request = "GetSlicesRequest"
    get_reservations_request = "GetReservationsRequest"
    get_delegations = "GetDelegations"
    get_reservation_units_request = "GetReservationUnitsRequest"
    get_unit_request = "GetUnitRequest"
    get_broker_query_model_request = "GetBrokerQueryModelRequest"
    get_actors_request = "GetActorsRequest"
    get_sites_request = "GetSitesRequest"

    result_slice = "ResultSlice"
    result_reservation = "ResultReservation"
    result_delegation = "ResultDelegation"
    result_reservation_state = "ResultReservationState"
    result_strings = "ResultStrings"
    result_string = "ResultString"
    result_units = "ResultUnits"
    result_sites = "ResultSites"
    result_proxy = "ResultProxy"
    result_broker_query_model = "ResultBrokerQueryModel"
    result_actor = "ResultActor"
    result_poa = "ResultPoa"

    def __init__(self, *, message_id: str = None, name: str = None, callback_topic: str = None, kafka_error: str = None,
                 id_token: str = None):
        # Unique id used to track produce request success/failures.
        # Do *not* include in the serialized object.
        self.id = uuid4()
        self.message_id = message_id
        if message_id is None:
            self.message_id = uuid4().__str__()
        self.name = name
        self.callback_topic = callback_topic
        self.kafka_error = kafka_error
        self.id_token = id_token
        self.attempt = 1

    def increment_attempt(self):
        self.attempt += 1

    def attempts(self) -> int:
        return self.attempt

    def to_dict(self) -> dict:
        """
        The Avro Python library does not support code generation.
        For this reason we must provide a dict representation of our class for serialization.
        :return dict representing the class
        """
        if not self.validate():
            raise MessageBusException(Constants.ERROR_INVALID_ARGUMENTS)

        result = self.__dict__.copy()

        for k in self.__dict__:
            if result[k] is None or k == Constants.ID or k == Constants.ATTEMPT:
                result.pop(k)
            elif isinstance(result[k], AbcObjectAvro):
                result[k] = result[k].to_dict()
            elif k in [Constants.VCPU_CPU_MAP, Constants.NODE_SET, Constants.KEYS, Constants.BDF]:
                v = result[k]
                if v is not None:
                    result[k] = pickle.dumps(v)
            elif isinstance(result[k], list):
                temp = []
                for elem in result[k]:
                    if isinstance(elem, AbcObjectAvro):
                        temp.append(elem.to_dict())
                    else:
                        temp.append(elem)
                result[k] = temp

        return result

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.AUTH:
                    self.__dict__[k] = AuthAvro()
                    self.__dict__[k].from_dict(value=v)
                else:
                    self.__dict__[k] = v

    def get_message_id(self) -> str:
        """
        Returns the message_id
        """
        return self.message_id

    def get_message_name(self) -> str:
        """
        Returns the message name
        """
        return self.name

    def get_callback_topic(self) -> str:
        """
        Returns the callback topic
        """
        return self.callback_topic

    def get_id(self) -> str:
        """
        Returns the id
        """
        return self.id.__str__()

    def get_id_token(self) -> str:
        """
        Returns id token
        """
        return self.id_token

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.get_message_name() is None or self.get_message_id() is None or self.get_id() is None:
            ret_val = False
        return ret_val

    def set_kafka_error(self, kafka_error):
        self.kafka_error = kafka_error

    def get_kafka_error(self):
        return self.kafka_error

    def __str__(self):
        d = self.__dict__.copy()
        for k in self.__dict__:
            if d[k] is None or k == "id":
                d.pop(k)
        if len(d) == 0:
            return ''
        ret = "{ "
        for i, v in d.items():
            ret = f"{ret} {i}: {v},"
        return ret[:-1] + "}"

    def __eq__(self, other):
        assert isinstance(other, AbcMessageAvro)
        for f, v in self.__dict__.items():
            if f == Constants.ID:
                continue
            if v != other.__dict__[f]:
                return False
        return True



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)

"""
Defines AvroConsumer API class which exposes interface for various consumer functions
"""
import importlib
import logging
import re
import time
import traceback
from typing import List

from confluent_kafka.avro import AvroConsumer, SerializerError
from confluent_kafka.cimpl import KafkaError, TopicPartition

from fabric_mb.message_bus.abc_mb_api import ABCMbApi
from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro


class AvroConsumerApi(ABCMbApi):
    """
    This class implements the Interface for Kafka consumer carrying Avro messages.
    It is expected that the users would extend this class and override handle_message function.
    """
    def __init__(self, *, consumer_conf: dict, key_schema_location, value_schema_location: str,
                 topics: List[str], batch_size: int = 5, logger: logging.Logger = None,
                 poll_timeout: int = 250):
        super(AvroConsumerApi, self).__init__(logger=logger)

        self.key_schema = self.load_schema(schema_file=key_schema_location)
        self.value_schema = self.load_schema(schema_file=value_schema_location)

        consumer_conf['on_commit'] = self.commit_completed
        self.consumer = AvroConsumer(consumer_conf, reader_key_schema=self.key_schema,
                                     reader_value_schema=self.value_schema)
        self.running = True
        self.topics = topics
        self.batch_size = batch_size
        self.poll_timeout = float(poll_timeout/1000)
        self.enable_auto_commit = consumer_conf.get("enable.auto.commit", True)

    def shutdown(self):
        """
        Shutdown the consumer
        :return:
        """
        self.logger.debug("Trigger shutdown")
        self.running = False

    @staticmethod
    def _create_instance(*, module_name: str, class_name: str):
        module = importlib.import_module(module_name)
        class_ = getattr(module, class_name)
        return class_()

    @staticmethod
    def _create_instance_with_params(*, module_name: str, class_name: str):
        module = importlib.import_module(module_name)
        class_ = getattr(module, class_name)
        return class_

    def process_message(self, topic: str, key: dict, value: dict):
        """
        Process the incoming message. Must be overridden in the derived class
        :param topic: topic name
        :param key: incoming message key
        :param value: incoming message value
        :return:
        """
        self.logger.debug("KAFKA: Message received for topic " + topic)
        self.logger.debug("KAFKA: Key = {}".format(key))
        self.logger.debug("KAFKA: Value = {}".format(value))
        class_name = value.get('name', None) + 'Avro'
        self.logger.debug("KAFKA: class_name = {}".format(class_name))
        module_name = f"fabric_mb.message_bus.messages.{re.sub(r'(?<!^)(?=[A-Z])', '_', class_name).lower()}"
        self.logger.debug(f"KAFKA: module_name = {module_name}")

        message = self._create_instance(module_name=module_name, class_name=class_name)
        message.from_dict(value)

        self.handle_message(message=message)

    def handle_message(self, message: AbcMessageAvro):
        """
        Handle incoming message; must be overridden by the derived class
        :param message: incoming message
        """
        print(message)

    def consume(self):
        """
            Consume records unless shutdown triggered. Using synchronous commit after a message batch.
        """
        self.consumer.subscribe(self.topics)

        msg_count = 0
        offsets = []
        while self.running:
            try:
                msg = self.consumer.poll(timeout=self.poll_timeout)

                # There were no messages on the queue, continue polling
                if msg is None:
                    continue

                if msg.error():
                    self.logger.error(f"KAFKA: Consumer error: {msg.error()}")
                    if msg.error().code() == KafkaError._PARTITION_EOF:
                        # End of partition event
                        self.logger.error(f"KAFKA: {msg.topic()} {msg.partition} reached end at offset [{msg.offset()}]")
                    elif msg.error():
                        self.logger.error(f"KAFKA: Consumer error: {msg.error()}")
                        continue

                # Retrieve current offset and high-water mark
                partition = msg.partition()
                topic = msg.topic()
                current_offset = msg.offset()
                offsets.append(TopicPartition(topic=topic, partition=partition, offset=current_offset + 1))
                low_mark, highwater_mark = self.consumer.get_watermark_offsets(TopicPartition(topic=topic,
                                                                                              partition=partition))

                # Calculate lag
                lag = highwater_mark - current_offset

                self.logger.info(
                    f"Partition {partition}: Current Offset={current_offset}, Highwater Mark={highwater_mark}, "
                    f"Lag={lag}, Current Message: {msg.value().get('name')}")
                begin = time.time()
                self.process_message(msg.topic(), msg.key(), msg.value())
                self.logger.info(f"KAFKA PROCESS TIME: {time.time() - begin:.0f}")

                if not self.enable_auto_commit:
                    msg_count += 1
                    if msg_count % self.batch_size == 0:
                        self.consumer.commit(offsets=offsets)
                        offsets.clear()

            except SerializerError as e:
                # Report malformed record, discard results, continue polling
                self.logger.error(f"KAFKA: Message deserialization failed {e}")
                self.logger.error(traceback.format_exc())
                continue
            except KeyboardInterrupt:
                break
            except Exception as e:
                self.logger.error(f"KAFKA: consumer error: {e}")
                self.logger.error(traceback.format_exc())

        self.logger.info("KAFKA: Shutting down consumer..")
        # Commit all the messages processed if any pending
        if len(offsets):
            self.consumer.commit(offsets=offsets, asynchronous=False)
        self.consumer.close()
        self.logger.info("KAFKA: Consumer Shutting down complete..")

    def commit_completed(self, err, partitions):
        if err:
            self.logger.error(f"KAFKA: commit failure: {err}")
        else:
            self.logger.debug(f"KAFKA: Committed partition offsets: {partitions}")


#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import pickle

from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro
from fabric_mb.message_bus.messages.constants import Constants
from fabric_mb.message_bus.messages.ticket import Ticket
from fabric_mb.message_bus.messages.unit_avro import UnitAvro


class ResourceSetAvro(AbcObjectAvro):
    """
    Implements Avro representation of a Reservation Resource Set
    """
    def __init__(self):
        self.units = None
        self.type = None
        self.sliver = None
        self.ticket = None
        self.unit_set = None

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.TICKET:
                    self.__dict__[k] = Ticket()
                    self.__dict__[k].from_dict(value=v)
                elif k == Constants.UNIT_SET:
                    self.__dict__[k] = []
                    for u in v:
                        uu = UnitAvro()
                        uu.from_dict(value=u)
                        self.__dict__[k].append(uu)
                elif k == Constants.SLIVER:
                    if v is not None:
                        self.__dict__[k] = pickle.loads(v)
                    else:
                        self.__dict__[k] = v
                else:
                    self.__dict__[k] = v

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.units is None or self.type is None:
            ret_val = False
        return ret_val

    def set_sliver(self, sliver):
        self.sliver = sliver

    def get_sliver(self):
        return self.sliver



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
"""
Defines Admin API class which exposes interface for various admin client functions
"""
import logging
import time

from confluent_kafka.admin import AdminClient, NewTopic, NewPartitions

from fabric_mb.message_bus.abc_mb_api import ABCMbApi


class AdminApi(ABCMbApi):
    """
    Implements interface for Admin APIs
    """
    def __init__(self, *, conf: dict, logger: logging.Logger = None):
        super().__init__(logger=logger)
        self.admin_client = AdminClient(conf)

    def create_topics(self, topics, num_partitions: int = 3, replication_factor: int = 1):
        """
            Create a list of topics
            :param topics: list of topics to be created
            :param num_partitions: number of partitions for the topic
            :param replication_factor: replication factor
            :return:
        """

        new_topics = [NewTopic(topic, num_partitions=num_partitions,
                               replication_factor=replication_factor) for topic in topics]

        # Call create_topics to asynchronously create topics, a dict
        # of <topic,future> is returned.
        fs = self.admin_client.create_topics(new_topics)

        # Wait for operation to finish.
        # Timeouts are preferably controlled by passing request_timeout=15.0
        # to the create_topics() call.
        # All futures will finish at the same time.
        for topic, f in fs.items():
            try:
                f.result()  # The result itself is None
                self.logger.debug(f"Topic {topic} created")
            except Exception as e:
                self.logger.error(f"Failed to create topic {topic}: {e}")

    def delete_topics(self, topics):
        """
            delete list of topics
            :param topics: list of topics
            :return:
        """

        # Call delete_topics to asynchronously delete topics, a future is returned.
        # By default this operation on the broker returns immediately while
        # topics are deleted in the background. But here we give it some time (30s)
        # to propagate in the cluster before returning.
        #
        # Returns a dict of <topic,future>.
        fs = self.admin_client.delete_topics(topics, operation_timeout=30)

        # Wait for operation to finish.
        for topic, f in fs.items():
            try:
                f.result()  # The result itself is None
                self.logger.debug(f"Topic {topic} deleted".format(topic))
            except Exception as e:
                self.logger.error(f"Failed to delete topic {topic}: {e}")

    def create_partitions(self, topic_partitions):
        """
            create partitions for list of topics
            :param topic_partitions: list of tuples (topic, number of partitions)
            :return:
        """

        new_parts = [NewPartitions(topic, int(new_total_count)) for
                     topic, new_total_count in topic_partitions]

        # Try switching validate_only to True to only validate the operation
        # on the broker but not actually perform it.
        fs = self.admin_client.create_partitions(new_parts, validate_only=False)

        # Wait for operation to finish.
        for topic, f in fs.items():
            try:
                f.result()  # The result itself is None
                self.logger.debug(f"Additional partitions created for topic {topic}")
            except Exception as e:
                self.logger.error(f"Failed to add partitions to topic {topic}: {e}")

    def list_topics(self, timeout: int = 10) -> list:
        """
            list topics and cluster metadata
            :param type: list topics or brokers or all; allowed values (all|topics|brokers)
            :param timeout: timeout in ms
            :return:
        """
        result = []
        md = self.admin_client.list_topics(timeout=timeout)

        for t in iter(md.topics.values()):
            result.append(str(t))

        return result


if __name__ == '__main__':

    # create admin client
    conf = {'metadata.broker.list': 'localhost:19092',
            'security.protocol': 'SSL',
            'ssl.ca.location': '../../secrets/snakeoil-ca-1.crt',
            'ssl.key.location': '../../secrets/kafkacat.client.key',
            'ssl.key.password': 'confluent',
            'ssl.certificate.location': '../../secrets/kafkacat-ca1-signed.pem'}
    api = AdminApi(conf=conf)
    test_topics = ['topic1', "topic2"]

    # create topics
    api.create_topics(test_topics)
    test_topic_partitions = [('topic1', 4), ("topic2", 4)]

    # create partitions
    api.create_partitions(test_topic_partitions)
    time.sleep(5)

    # delete partitions
    api.delete_topics(test_topics)



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from typing import Dict, List

from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro
from fabric_mb.message_bus.messages.constants import Constants
from fabric_mb.message_bus.messages.site_avro import SiteAvro


class MaintenanceRequestAvro(AbcMessageAvro):
    """
    Implements Avro representation of an Maintenance Request Message
    """
    def __init__(self, *, properties: Dict[str, str] = None, actor_guid: str = None, callback_topic: str = None,
                 id_token: str = None, message_id: str = None, sites: List[SiteAvro] = None):
        super(MaintenanceRequestAvro, self).__init__(callback_topic=callback_topic, id_token=id_token,
                                                     name=AbcMessageAvro.maintenance_request, message_id=message_id)
        self.actor_guid = actor_guid
        self.properties = properties
        self.sites = sites

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.SITES:
                    self.from_dict_sites(value=v)
                else:
                    self.__dict__[k] = v

    def from_dict_sites(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            for s in value:
                site_obj = SiteAvro()
                site_obj.from_dict(s)
                if self.sites is None:
                    self.sites = []
                self.sites.append(site_obj)

    def set_properties(self, properties: Dict[str, str]):
        self.properties = properties

    def get_properties(self) -> Dict[str, str]:
        return self.properties

    def set_sites(self, sites: List[SiteAvro]):
        self.sites = sites

    def get_sites(self) -> List[SiteAvro]:
        return self.sites



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
"""
Defines AvroProducer API class which exposes interface for various producer functions
"""
import logging
import threading
import traceback

from confluent_kafka.avro import AvroProducer

from fabric_mb.message_bus.abc_mb_api import ABCMbApi
from fabric_mb.message_bus.admin import AdminApi
from fabric_mb.message_bus.messages.auth_avro import AuthAvro
from fabric_mb.message_bus.messages.query_avro import QueryAvro
from fabric_mb.message_bus.messages.query_result_avro import QueryResultAvro
from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro


class AvroProducerApi(ABCMbApi):
    """
        This class implements the Interface for Kafka producer carrying Avro messages.
        It is expected that the users would extend this class and override on_delivery function.
    """
    def __init__(self, *, producer_conf: dict, key_schema_location, value_schema_location: str,
                 logger: logging.Logger = None, retries: int = 3):
        """
            Initialize the Producer API
            :param producer_conf: configuration e.g:
                        {'bootstrap.servers': localhost:9092,
                        'schema.registry.url': http://localhost:8083}
            :param key_schema_location: AVRO schema location for the key
            :param value_schema_location: AVRO schema location for the value
        """
        super(AvroProducerApi, self).__init__(logger=logger)
        self.lock = threading.Lock()
        self.key_schema = self.load_schema(schema_file=key_schema_location)
        self.value_schema = self.load_schema(schema_file=value_schema_location)
        self.producer = AvroProducer(producer_conf, default_key_schema=self.key_schema,
                                     default_value_schema=self.value_schema)
        self.retry_attempts = retries

    def load_schema(self, schema_file: str):
        try:
            from confluent_kafka import avro
            file = open(schema_file, "r")
            schema_bytes = file.read()
            file.close()
            return avro.loads(schema_bytes)
        except Exception as e:
            self.logger.error(f"Exception occurred while loading the schema: {schema_file}: {e}")
            self.logger.error(traceback.format_exc())

    def set_logger(self, logger):
        """
        Set logger
        :param logger: logger
        """
        self.logger = logger

    def delivery_report(self, err, msg, obj: AbcMessageAvro):
        """
            Handle delivery reports served from producer.poll.
            This callback takes an extra argument, obj.
            This allows the original contents to be included for debugging purposes.
        """
        if err is not None:
            self.logger.error(f"KAFKA: Message Delivery Failure! Error [{err}] MsgId: [{obj.id}] "
                              f"Msg Name: [{obj.name}]")
            obj.set_kafka_error(kafka_error=err)
        else:
            self.logger.debug(f"KAFKA: Message Delivery Successful! MsgId: [{obj.id}] Msg Name: [{obj.name}] "
                              f"Topic: [{msg.topic()}] Partition [{msg.partition()}] Offset [{msg.offset()}]")

    def produce(self, topic, record: AbcMessageAvro) -> bool:
        """
            Produce records for a specific topic
            :param topic: topic to which messages are written to
            :param record: record/message to be written
            :return:
        """
        try:
            self.logger.debug(f"KAFKA: Record type={type(record)}")
            self.logger.debug(f"KAFKA: Producing key {record.get_id()} to topic {topic}.")
            self.logger.debug(f"KAFKA: Producing record {record.to_dict()} to topic {topic}.")

            self.lock.acquire()
            # Pass the message asynchronously
            self.producer.produce(topic=topic, key=record.get_id(), value=record.to_dict(),
                                  callback=lambda err, msg, obj=record: self.delivery_report(err, msg, obj))
            return True
        except ValueError as ex:
            self.logger.error("KAFKA: Invalid input, discarding record...")
            self.logger.error(f"KAFKA: Exception occurred {ex}")
            self.logger.error(traceback.format_exc())
        except Exception as ex:
            self.logger.error(f"KAFKA: Exception occurred {ex}")
            self.logger.error(traceback.format_exc())
        finally:
            if self.lock.locked():
                self.lock.release()
        return False

    def poll(self, timeout: float = 0.0):
        """
            Poll for delivery callbacks
            :param timeout: timeout
            :return:
        """
        try:
            self.lock.acquire()
            return self.producer.poll(timeout=timeout)
        finally:
            self.lock.release()

    def flush(self, timeout: float = None):
        """
            Flush all pending writes
            :param timeout: timeout
            :return:
        """
        try:
            self.lock.acquire()
            self.producer.flush()
        finally:
            self.lock.release()


if __name__ == '__main__':
    # Create Admin API object
    conf = {'metadata.broker.list': 'localhost:19092',
            'security.protocol': 'SSL',
            'ssl.ca.location': '../../secrets/snakeoil-ca-1.crt',
            'ssl.key.location': '../../secrets/kafkacat.client.key',
            'ssl.key.password': 'confluent',
            'ssl.certificate.location': '../../secrets/kafkacat-ca1-signed.pem',
            'request.timeout.ms': 120}

    api = AdminApi(conf=conf)
    topics = ['topic1']

    # create topics
    api.create_topics(topics)

    test_key_schema = "schema/key.avsc"
    test_val_schema = "schema/message.avsc"

    producer_conf = conf.copy()
    producer_conf['schema.registry.url'] = "http://localhost:8081"

    # create a producer
    producer = AvroProducerApi(producer_conf=producer_conf, key_schema_location=test_key_schema,
                               value_schema_location=test_val_schema)

    # produce message to topics
    id_token = 'id_token'

    auth = AuthAvro()
    auth.guid = "testguid"
    auth.name = "testactor"
    auth.oidc_sub_claim = "test-oidc"

    query = QueryAvro()
    query.message_id = "msg1"
    query.callback_topic = "topic"
    query.properties = {"abc": "def"}
    query.auth = auth
    query.id_token = id_token
    producer.produce("topic1", query)

    query_result = QueryResultAvro()
    query_result.message_id = "msg2"
    query_result.request_id = "req2"
    query_result.properties = {"abc": "def"}
    query_result.auth = auth
    producer.produce("topic1", query_result)

    # delete topics
    api.delete_topics(topics)



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import pickle
from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro


class UnitAvro(AbcObjectAvro):
    """
    Implements Avro representation of a Unit
    """
    def __init__(self):
        self.rtype = None
        self.parent_id = None
        self.state = None
        self.sequence = 0
        self.properties = None
        self.reservation_id = None
        self.slice_id = None
        self.actor_id = None
        self.sliver = None

    def get_unit_id(self) -> str:
        return self.reservation_id

    def get_properties(self) -> dict:
        """
        Return properties
        """
        return self.properties

    def get_resource_type(self) -> str:
        return self.rtype

    def get_parent_id(self) -> str:
        return self.parent_id

    def get_state(self) -> int:
        return self.state

    def get_sequence(self) -> int:
        return self.sequence

    def get_reservation_id(self) -> str:
        return self.reservation_id

    def get_slice_id(self) -> str:
        return self.slice_id

    def get_actor_id(self) -> str:
        return self.actor_id

    def set_properties(self, value: dict):
        """
        Set properties
        @param value value
        """
        self.properties = value

    def set_unit_id(self, uid: str):
        self.reservation_id = uid

    def set_resource_type(self, rtype: str):
        self.rtype = rtype

    def set_parent_id(self, parent_id: str):
        self.parent_id = parent_id

    def set_state(self, state: int):
        self.state = state

    def set_sequence(self, sequence: int):
        self.sequence = sequence

    def set_reservation_id(self, reservation_id: str):
        self.reservation_id = reservation_id

    def set_slice_id(self, slice_id: str):
        self.slice_id = slice_id

    def set_actor_id(self, actor_id: str):
        self.actor_id = actor_id

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True

        if self.properties is None or self.reservation_id is None:
            ret_val = False

        return ret_val

    def set_sliver(self, sliver):
        self.sliver = sliver

    def get_sliver(self):
        return self.sliver



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from typing import List
from uuid import uuid4

from fabric_mb.message_bus.message_bus_exception import MessageBusException
from fabric_mb.message_bus.messages.actor_avro import ActorAvro
from fabric_mb.message_bus.messages.constants import Constants
from fabric_mb.message_bus.messages.delegation_avro import DelegationAvro
from fabric_mb.message_bus.messages.lease_reservation_avro import LeaseReservationAvro
from fabric_mb.message_bus.messages.lease_reservation_state_avro import LeaseReservationStateAvro
from fabric_mb.message_bus.messages.broker_query_model_avro import BrokerQueryModelAvro
from fabric_mb.message_bus.messages.poa_info_avro import PoaInfoAvro
from fabric_mb.message_bus.messages.proxy_avro import ProxyAvro
from fabric_mb.message_bus.messages.reservation_mng import ReservationMng
from fabric_mb.message_bus.messages.reservation_state_avro import ReservationStateAvro
from fabric_mb.message_bus.messages.result_avro import ResultAvro
from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro
from fabric_mb.message_bus.messages.site_avro import SiteAvro
from fabric_mb.message_bus.messages.slice_avro import SliceAvro
from fabric_mb.message_bus.messages.ticket_reservation_avro import TicketReservationAvro
from fabric_mb.message_bus.messages.unit_avro import UnitAvro


class ResultRecordList(AbcMessageAvro):
    """
    Implements Avro representation of a Result Record List
    """
    def __init__(self):
        super(ResultRecordList, self).__init__()
        self.status = None
        self.slices = None
        self.reservations = None
        self.reservation_states = None
        self.units = None
        self.proxies = None
        self.model = None
        self.actors = None
        self.delegations = None
        self.sites = None
        self.poas = None

    def from_dict_slices(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            for s in value:
                slice_obj = SliceAvro()
                slice_obj.from_dict(s)
                if self.slices is None:
                    self.slices = []
                self.slices.append(slice_obj)

    def from_dict_reservations(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            for s in value:
                res_obj = None
                if s.get(Constants.NAME) == LeaseReservationAvro.__name__:
                    res_obj = LeaseReservationAvro()
                elif s.get(Constants.NAME) == TicketReservationAvro.__name__:
                    res_obj = TicketReservationAvro()
                else:
                    res_obj = ReservationMng()
                res_obj.from_dict(s)
                if self.reservations is None:
                    self.reservations = []
                self.reservations.append(res_obj)

    def from_dict_reservation_states(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            self.reservation_states = []
            for rs in value:
                rs_state = None
                if rs.get(Constants.NAME) == LeaseReservationStateAvro.__name__:
                    rs_state = LeaseReservationStateAvro()
                else:
                    rs_state = ReservationStateAvro()
                rs_state.from_dict(rs)
                self.reservation_states.append(rs_state)

    def from_dict_units(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            self.units = []
            for u in value:
                unit = UnitAvro()
                unit.from_dict(u)
                self.units.append(unit)

    def from_dict_proxies(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            for p in value:
                proxy_obj = ProxyAvro()
                proxy_obj.from_dict(p)
                if self.proxies is None:
                    self.proxies = []
                self.proxies.append(proxy_obj)

    def from_dict_model(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            self.model = BrokerQueryModelAvro()
            self.model.from_dict(value)

    def from_dict_actors(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            for s in value:
                actor_obj = ActorAvro()
                actor_obj.from_dict(s)
                if self.actors is None:
                    self.actors = []
                self.actors.append(actor_obj)

    def from_dict_delegations(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            for d in value:
                del_obj = DelegationAvro()
                del_obj.from_dict(d)
                if self.delegations is None:
                    self.delegations = []
                self.delegations.append(del_obj)

    def from_dict_sites(self, value: list):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            self.sites = []
            for s in value:
                site = SiteAvro()
                site.from_dict(s)
                self.sites.append(site)

    def from_dict_poas(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value is not None:
            self.poas = []
            for p in value:
                poa = PoaInfoAvro()
                poa.from_dict(p)
                self.poas.append(poa)

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.STATUS:
                    self.status = ResultAvro()
                    self.status.from_dict(value=v)
                elif k == Constants.SLICES:
                    self.from_dict_slices(value=v)
                elif k == Constants.RESERVATIONS:
                    self.from_dict_reservations(value=v)
                elif k == Constants.RESERVATION_STATES:
                    self.from_dict_reservation_states(value=v)
                elif k == Constants.UNITS:
                    self.from_dict_units(value=v)
                elif k == Constants.PROXIES:
                    self.from_dict_proxies(value=v)
                elif k == Constants.MODEL:
                    self.from_dict_model(value=v)
                elif k == Constants.ACTORS:
                    self.from_dict_actors(value=v)
                elif k == Constants.DELEGATIONS:
                    self.from_dict_delegations(value=v)
                elif k == Constants.SITES:
                    self.from_dict_sites(value=v)
                elif k == Constants.POAS:
                    self.from_dict_poas(value=v)
                else:
                    self.__dict__[k] = v

    def set_status(self, status: ResultAvro):
        """
        Set status
        """
        self.status = status

    def get_status(self) -> ResultAvro:
        """
        Return status
        """
        return self.status

    def get_slices(self) -> List[SliceAvro]:
        """
        Return slices
        """
        return self.slices

    def get_reservations(self) -> List[ReservationMng]:
        """
        Return reservations
        """
        return self.reservations

    def get_reservation_states(self) -> List[ReservationStateAvro]:
        """
        Return reservation states
        """
        return self.reservation_states

    def get_units(self) -> List[UnitAvro]:
        """
        Return units
        """
        return self.units

    def get_proxies(self) -> List[ProxyAvro]:
        """
        Return proxies
        """
        return self.proxies

    def get_model(self) -> BrokerQueryModelAvro:
        """
        Return model
        """
        return self.model

    def get_actors(self) -> List[ActorAvro]:
        """
        Return actors
        """
        return self.actors

    def get_delegations(self) -> List[DelegationAvro]:
        """
        Return delegations
        """
        return self.delegations

    def get_sites(self) -> List[SiteAvro]:
        """
        Return sites
        """
        return self.sites

    def get_poas(self) -> List[PoaInfoAvro]:
        """
        Return Poa Info
        """
        return self.poas

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = super().validate()
        if self.status is None:
            ret_val = False
        return ret_val



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro
from fabric_mb.message_bus.messages.constants import Constants
from fabric_mb.message_bus.messages.term_avro import TermAvro


class ResourceTicketAvro(AbcObjectAvro):
    def __init__(self):
        self.guid = None
        self.term = None
        self.units = 0
        self.properties = None
        self.type = None
        self.issuer = None
        self.holder = None

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.TERM:
                    self.__dict__[k] = TermAvro()
                    self.__dict__[k].from_dict(value=v)
                else:
                    self.__dict__[k] = v

    def get_guid(self) -> str:
        return self.guid

    def get_units(self) -> int:
        return self.units

    def get_term(self) -> TermAvro:
        return self.term

    def get_properties(self) -> dict:
        return self.properties

    def get_type(self) -> str:
        return self.type

    def get_issuer(self) -> str:
        return self.issuer

    def get_holder(self) -> str:
        return self.holder

    def set_guid(self, guid: str):
        self.guid = guid

    def set_units(self, units: int):
        self.units = units

    def set_term(self, term: TermAvro):
        self.term = term

    def set_properties(self, properties: dict):
        self.properties = properties

    def set_type(self, rtype: str):
        self.type = rtype

    def set_issuer(self, issuer: str):
        self.issuer = issuer

    def set_holder(self, holder: str):
        self.holder = holder

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.guid is None or self.units is None or self.type is None:
            ret_val = False
        return ret_val



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro
from fabric_mb.message_bus.messages.constants import Constants
from fabric_mb.message_bus.messages.resource_set_avro import ResourceSetAvro
from fabric_mb.message_bus.messages.slice_avro import SliceAvro
from fabric_mb.message_bus.messages.term_avro import TermAvro


class ReservationAvro(AbcObjectAvro):
    """
    Implements Avro representation of a Reservation
    """
    def __init__(self):
        self.reservation_id = None
        self.slice = None
        self.resource_set = None
        self.term = None
        self.sequence = None

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                if k == Constants.SLICE:
                    self.__dict__[k] = SliceAvro()
                    self.__dict__[k].from_dict(value=v)
                elif k == Constants.TERM:
                    self.__dict__[k] = TermAvro()
                    self.__dict__[k].from_dict(value=v)
                elif k == Constants.RESOURCE_SET:
                    self.__dict__[k] = ResourceSetAvro()
                    self.__dict__[k].from_dict(value=v)
                else:
                    self.__dict__[k] = v

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.reservation_id is None or self.slice is None or self.term is None:
            ret_val = False
        return ret_val



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
import pickle
from fabric_mb.message_bus.messages.abc_object_avro import AbcObjectAvro


class ReservationMng(AbcObjectAvro):
    """
    Implements Avro representation of a Reservation returned from Management Interface
    """
    def __init__(self):
        self.name = self.__class__.__name__
        self.reservation_id = None
        self.slice_id = None
        self.start = None
        self.end = None
        self.requested_end = None
        self.closed_at = None
        self.rtype = None
        self.units = None
        self.state = None
        self.pending_state = None
        self.local = None
        self.config = None
        self.request = None
        self.resource = None
        self.notices = None
        self.sliver = None

    def print(self):
        """
        Prints ReservationMng
        """
        print("")
        print(f"Reservation ID: {self.reservation_id} Slice ID: {self.slice_id}")
        if self.rtype is not None or self.notices is not None:
            print(f"Resource Type: {self.rtype} Notices: {self.notices}")

        if self.start is not None or self.end is not None or self.requested_end is not None:
            print(f"Start: {self.start} End: {self.end} Requested End: {self.requested_end}")

        if self.units is not None or self.state is not None or self.pending_state is not None:
            print(f"Units: {self.units} State: {self.state} Pending State: {self.pending_state}")

        if self.local is not None:
            print(f"Local Properties: {self.local}")
        if self.config is not None:
            print(f"Config Properties: {self.config}")
        if self.request is not None:
            print(f"Request Properties: {self.request}")
        if self.resource is not None:
            print(f"Resource Properties: {self.resource}")

        print("")

    def get_reservation_id(self) -> str:
        """
        Returns Reservation ID
        @return reservation id
        """
        return self.reservation_id

    def set_reservation_id(self, value: str):
        """
        Set reservation id
        @param value value
        """
        self.reservation_id = value

    def get_slice_id(self) -> str:
        """
        Return Slice Id
        @return slice id
        """
        return self.slice_id

    def set_slice_id(self, value: str):
        """
        Set slice id
        @param value value
        """
        self.slice_id = value

    def get_start(self) -> int:
        """
        Return Start Lease Time
        @return start time
        """
        return self.start

    def set_start(self, value: int):
        """
        Set Start lease time
        @param value value
        """
        self.start = value

    def get_end(self) -> int:
        """
        Return End lease time
        @return end time
        """
        return self.end

    def set_end(self, value: int):
        """
        Set end lease time
        @param value value
        """
        self.end = value

    def get_requested_end(self) -> int:
        """
        Return requested end lease time
        @return requested end lease time
        """
        return self.requested_end

    def set_requested_end(self, value: int):
        """
        Set requested lease end time
        @param value value
        """
        self.requested_end = value

    def get_closed_at(self) -> int:
        """
        Return closed at time
        @return closed at time in milliseconds
        """
        return self.closed_at

    def set_closed_at(self, value: int):
        """
        Set closed at time
        @param value value in milliseconds
        """
        self.closed_at = value

    def get_resource_type(self) -> str:
        """
        Return resource type
        @return resource type
        """
        return self.rtype

    def set_resource_type(self, value: str):
        """
        Set resource type
        @param value value
        """
        self.rtype = value

    def get_units(self) -> int:
        """
        Return Units
        @param return unitds
        """
        return self.units

    def set_units(self, value: int):
        """
        Set units
        @param value value
        """
        self.units = value

    def get_state(self) -> int:
        """
        Return reservation state
        @return reservation state
        """
        return self.state

    def set_state(self, value: int):
        """
        Set reservation state
        @param value value
        """
        self.state = value

    def get_pending_state(self) -> int:
        """
        Return Pending State
        @return pending state
        """
        return self.pending_state

    def set_pending_state(self, value: int):
        """
        Set pending state
        @param value value
        """
        self.pending_state = value

    def get_local_properties(self) -> dict:
        """
        Return local properties
        @return local properties
        """
        return self.local

    def set_local_properties(self, value: dict):
        """
        Set local properties
        @param value value
        """
        self.local = value

    def get_config_properties(self) -> dict:
        """
        Return config properties
        @return config properties
        """
        return self.config

    def set_config_properties(self, value: dict):
        """
        Set config properties
        @param value value
        """
        self.config = value

    def get_request_properties(self) -> dict:
        """
        Return request properties
        @return request properties
        """
        return self.request

    def set_request_properties(self, value: dict):
        """
        Set request properties
        @param value value
        """
        self.request = value

    def get_resource_properties(self) -> dict:
        """
        Return resource properties
        @return resource properties
        """
        return self.resource

    def set_resource_properties(self, value: dict):
        """
        Set resource properties
        @param value value
        """
        self.resource = value

    def get_notices(self) -> str:
        """
        Return notices
        @return notices
        """
        return self.notices

    def set_notices(self, value: str):
        """
        Set notices
        @param value value
        """
        self.notices = value

    def get_sliver(self):
        return self.sliver

    def set_sliver(self, sliver):
        self.sliver = sliver

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = True
        if self.reservation_id is None or self.rtype is None or self.notices is None:
            ret_val = False
        return ret_val



#!/usr/bin/env python3
# MIT License
#
# Copyright (c) 2020 FABRIC Testbed
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#
# Author: Komal Thareja (kthare10@renci.org)
from __future__ import annotations

from typing import List

from fabric_mb.message_bus.message_bus_exception import MessageBusException
from fabric_mb.message_bus.messages.abc_message_avro import AbcMessageAvro
from fabric_mb.message_bus.messages.constants import Constants
from fabric_mb.message_bus.messages.result_avro import ResultAvro


class ResultStringsAvro(AbcMessageAvro):
    """
    Implements Avro representation of a Result Message containing Strings
    """

    def __init__(self):
        super(ResultStringsAvro, self).__init__()
        self.name = AbcMessageAvro.result_strings
        self.status = None
        self.result = None

    def from_dict(self, value: dict):
        """
        The Avro Python library does not support code generation.
        For this reason we must provide conversion from dict to our class for de-serialization
        :param value: incoming message dictionary
        """
        if value[Constants.NAME] != AbcMessageAvro.result_strings:
            raise MessageBusException(Constants.ERROR_INVALID_MESSAGE)

        for k, v in value.items():
            if k in self.__dict__ and v is not None:
                self.__dict__[k] = v
                if k == Constants.STATUS:
                    self.status = ResultAvro()
                    self.status.from_dict(value=v)

    def get_status(self) -> ResultAvro:
        """
        Returns the status
        """
        return self.status

    def set_status(self, value: ResultAvro):
        """
        Set status
        @param value value
        """
        self.status = value

    def get_result(self) -> List[str]:
        """
        Returns the result
        """
        return self.result

    def validate(self) -> bool:
        """
        Check if the object is valid and contains all mandatory fields
        :return True on success; False on failure
        """
        ret_val = super().validate()
        if self.status is None:
            ret_val = False
        return ret_val
